#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <atari.h>

#define SIO_BUFFER_SIZE 128
#define SIO_NETWORK_DEV_ID 0x4E
#define SIO_TIMEOUT 0x0A
#define SIO_CONNECTION_ID 0x00
#define CR 0x0D
#define LF 0x0A

char sio_buffer[SIO_BUFFER_SIZE];
char status[5];

void init();

/* returns 0 if network is available, otherwise -1 */
int getNetworkStatus();

/* returns 0 if network is available and there are no errors related to the connection, otherwise -1 */
int getNetworkAndConnectionStatus(); 

/* returns 0 if open was successful, otherwise -1 */
int open(const char* url);

/* closes the connection */
void close();

/* reads (and returns) a line of text (over the connection) */
char* readLine();

/* writes a line of text (over the connection) and returns 0 if it was successful, otherwise -1*/
int writeLine(char* text);

/* returns use input */
char* getUserInput();

int main(void)
{
   char* fromServer = NULL;
   
   clrscr ();
   init();
   
   printf("KnockKnockClient\n");
   
   if(getNetworkStatus()==0)
   {
      do
      {
         printf("Enter IP Address and the port number\nfor example: 192.168.175.40:4444\n");
      }
      while(open(getUserInput()) != 0);
      
      while((fromServer = readLine()) != NULL)
      {
         printf("S: %s\n",fromServer);
         if(strcmp("Bye.",fromServer)==0)break;
         printf("C: ");
         if(writeLine(getUserInput()) != 0)break;
      }

      close();
   }
   else
   {
      printf("No network connection\n");
   }
   
   cgetc();
   
   return EXIT_SUCCESS;
}

void init()
{
   DCB.device = SIO_NETWORK_DEV_ID;
   DCB.unit = 1;
}

int getNetworkStatus()
{
   DCB.command = 0x53; /* GET STATUS */
   DCB.status = 0x40;  /* INPUT type operation*/
   DCB.buffer = sio_buffer;
   DCB.timeout = SIO_TIMEOUT;
   DCB.xfersize = 5;
   DCB.aux1 = 0;
   DCB.aux2 = 0;
   __asm__ ("jsr $E459");
   memcpy(status,sio_buffer,5);
   return ((DCB.status!=1) || (status[4]!=1))?-1:0;
}

int getNetworkAndConnectionStatus()
{
   getNetworkStatus();
   return ((DCB.status!=1) || (status[4]!=1) || (status[SIO_CONNECTION_ID] & 0x01))?-1:0;
}

int open(const char* url)
{
   strcpy(sio_buffer,url);
   DCB.command = 0x4F; /* OPEN */
   DCB.status = 0x80;  /* OUTPUT type operation*/
   DCB.buffer = sio_buffer;
   DCB.timeout = SIO_TIMEOUT;
   DCB.xfersize = strlen(sio_buffer);
   DCB.aux1 = SIO_CONNECTION_ID;
   DCB.aux2 = DCB.xfersize;
   __asm__ ("jsr $E459");
   return ((DCB.status!=1) || (getNetworkAndConnectionStatus()!=0))?-1:0;
}

void close()
{
   DCB.command = 0x43; /* CLOSE */
   DCB.status = 0x00;  /* IMMEDIATE type operation */
   DCB.buffer = sio_buffer;
   DCB.timeout = SIO_TIMEOUT;
   DCB.xfersize = 0;
   DCB.aux1 = SIO_CONNECTION_ID;
   DCB.aux2 = 0;
   __asm__ ("jsr $E459");
}

char* readLine()
{
   int char_read = 0;
   while(char_read < SIO_BUFFER_SIZE)
   {
      DCB.command = 0x52; /* READ */
      DCB.status = 0x40;  /* INPUT type operation*/
      DCB.buffer = &sio_buffer[char_read];
      DCB.timeout = SIO_TIMEOUT;
      DCB.xfersize = SIO_BUFFER_SIZE-char_read;
      DCB.aux1 = SIO_CONNECTION_ID;
      DCB.aux2 = SIO_BUFFER_SIZE-char_read-1; /* to be read over TCP/IP */
      __asm__ ("jsr $E459");
      char_read = char_read + sio_buffer[SIO_BUFFER_SIZE-1];
      if(sio_buffer[char_read-2]==CR && sio_buffer[char_read-1]==LF)
      {
         sio_buffer[char_read-2] = 0;
         return sio_buffer;
      }
      if(getNetworkAndConnectionStatus()!=0)break;
   }
   return NULL;
}

int writeLine(char* text)
{
   int ret_val = 0;   
   int text_length = strlen(text);
   
   if(text_length > SIO_BUFFER_SIZE-2)
   {
      // trim the line
      text_length = SIO_BUFFER_SIZE-2;
   }
   strncpy(sio_buffer,text,text_length);
   sio_buffer[text_length]   = CR;
   sio_buffer[text_length+1] = LF;
   
   DCB.command = 0x50; /* WRITE */
   DCB.status = 0x80;  /* OUTPUT type operation*/
   DCB.buffer = sio_buffer;
   DCB.timeout = SIO_TIMEOUT;
   DCB.xfersize = text_length+2;
   DCB.aux1 = SIO_CONNECTION_ID;
   DCB.aux2 = DCB.xfersize;
   __asm__ ("jsr $E459");
   
   return getNetworkAndConnectionStatus();
}

/*
** Get user input.
*/
char* getUserInput(void)
{
   unsigned len;
   fgets(sio_buffer, sizeof(sio_buffer), stdin);
   len = strlen(sio_buffer);
   if (len >= 1 && sio_buffer[len - 1] == '\n')
          sio_buffer[len - 1] = '\0';
   return sio_buffer;
}
